<?php
/**
 * @package   ats
 * @copyright Copyright (c)2011-2022 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU General Public License version 3, or later
 */

namespace Akeeba\Component\ATS\Administrator\Controller;

defined('_JEXEC') or die;

use Akeeba\Component\ATS\Administrator\Controller\Mixin\ControllerEvents;
use Akeeba\Component\ATS\Administrator\Controller\Mixin\GetRedirectionAware;
use Akeeba\Component\ATS\Administrator\Controller\Mixin\NewPostAware;
use Akeeba\Component\ATS\Administrator\Controller\Mixin\ReturnURLAware;
use Akeeba\Component\ATS\Administrator\Controller\Mixin\ReusableModels;
use Akeeba\Component\ATS\Administrator\Helper\Permissions;
use Akeeba\Component\ATS\Administrator\Model\PostModel;
use Akeeba\Component\ATS\Administrator\Table\PostTable;
use Akeeba\Component\ATS\Administrator\Table\TicketTable;
use Akeeba\Component\ATS\Administrator\View\Post\HtmlView;
use Exception;
use Joomla\CMS\Application\CMSApplication;
use Joomla\CMS\Form\FormFactoryInterface;
use Joomla\CMS\MVC\Controller\FormController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\Input\Input;

class PostController extends FormController
{
	use ControllerEvents;
	use ReusableModels
	{
		ReusableModels::getModel as reusableGetModel;
	}
	use ReturnURLAware;
	use NewPostAware;
	use GetRedirectionAware;

	/** @inheritdoc */
	protected $text_prefix = 'COM_ATS_POST';

	/** @inheritdoc */
	protected $view_list = 'tickets';

	/** @inheritdoc */
	protected $name = 'post';

	/**
	 * Flag set by TicketController to indicate we are saving a guest ticket (no permissions checked)
	 *
	 * @var   bool
	 * @since 5.0.0
	 */
	private $guestTicket = false;

	/** @inheritdoc */
	public function __construct($config = [], MVCFactoryInterface $factory = null, ?CMSApplication $app = null, ?Input $input = null, FormFactoryInterface $formFactory = null)
	{
		$this->guestTicket = (bool) ($config['guestTicket'] ?? false);

		parent::__construct($config, $factory, $app, $input, $formFactory);
	}

	/** @inheritdoc */
	public function getModel($name = 'Post', $prefix = '', $config = [])
	{
		if ($this->guestTicket)
		{
			$config['guestTicket'] = true;
		}

		return $this->reusableGetModel($name, $prefix, $config);
	}

	/** @inheritdoc */
	protected function allowAdd($data = [])
	{
		$ticket_id = $data['ticket_id'] ?? null;

		if (empty($ticket_id))
		{
			return false;
		}

		/** @var TicketTable $ticket */
		$ticket = $this->getModel()->getTable('Ticket', 'Administrator');

		if (!$ticket->load($ticket_id))
		{
			return false;
		}

		if ($this->guestTicket)
		{
			return true;
		}

		$canDo = Permissions::getTicketPrivileges($ticket);

		return $canDo['post'];
	}

	/** @inheritdoc */
	protected function allowEdit($data = [], $key = 'id')
	{
		$ticket_id = $data['ticket_id'] ?? null;
		$post_id   = $data['id'] ?? null;

		if (empty($ticket_id) && empty($post_id))
		{
			return false;
		}

		/** @var PostTable $post */
		$post = $this->getModel()->getTable('Post', 'Administrator');
		/** @var TicketTable $ticket */
		$ticket = $this->getModel()->getTable('Ticket', 'Administrator');

		if (!empty($ticket_id))
		{
			if (!$ticket->load($ticket_id))
			{
				return false;
			}
		}

		if (!empty($post_id))
		{
			if (!$post->load($post_id))
			{
				return false;
			}
		}

		if (empty($ticket_id) && !empty($post_id))
		{
			$ticket = $post->getTicket();
		}

		if (empty($ticket->id))
		{
			return false;
		}

		$canDo = Permissions::getPostPrivileges($post);

		return $canDo['edit'];
	}

	/**
	 * Runs after the save task (Save & Close button).
	 *
	 * Used to apply the return URL.
	 *
	 * @return  void
	 * @since   5.0.0
	 */
	protected function onAfterSave(): void
	{
		$this->applyReturnUrl();
	}

	/**
	 * Runs before the main display task.
	 *
	 * Used to communicate the return URL to the view.
	 *
	 * @return  void
	 * @throws  Exception
	 * @since   5.0.0
	 */
	protected function onBeforeMain(): void
	{
		/** @var HtmlView $view */
		$view            = $this->getView();
		$view->returnUrl = $this->getReturnUrl();
	}

	/**
	 * Called after we have successfully handled saving a post.
	 *
	 * Used to handle attachments, user tags and post notifications (e.g. emails).
	 *
	 * @param   BaseDatabaseModel|PostModel  $model      The model this controller acts upon
	 * @param   array                        $validData  Validated data being saved
	 *
	 * @return  void
	 * @throws  Exception
	 * @since   5.0.0
	 */
	protected function postSaveHook(BaseDatabaseModel $model, $validData = [])
	{
		// Make sure we have a post where we are attaching the uploads
		$postId = $model->getState('post.id', $validData['id'] ?? null);

		if (!$postId)
		{
			return;
		}

		/** @var PostTable $post */
		$post  = $model->getItem();
		$canDo = Permissions::getPostPrivileges($post);

		// Handle attachments if allowed
		if ($canDo['attachment'])
		{
			$this->handleAttachments($post);
		}

		// Handle user tags if allowed
		if ($canDo['admin'])
		{
			$this->handleUserTags($post);
		}

		// Send post notifications (emails etc)
		$newPost = $model->getState('post.isNew', true);
		$model->setState('post.isNew', null);
		$this->postNotifiable($post, $newPost);
	}
}